# -*- coding: utf-8 -*-
import numpy as np
from time import perf_counter

class World(object):   
    '''Stellt das Universum zu einem bestimmten Zeitpunkt dar.'''

    running = False
    paused = False
    duration = 0
    end_time = 0
       
    time = 0
    world_objects = {}
    
    input = None
    outputs = []
    
    def start(self, end_time):
        self.end_time = end_time
        self.duration = 0
        
        self.input.start()
        for output in self.outputs:
            output.start()
        
        self.running = True
        
        time_diff = end_time - self.time
        print('Starting simulation with {world_objects} objects for {time_diff} seconds (starting at {start_time}, ending at {end_time})...'.format(
                world_objects=len(self.world_objects), time_diff=time_diff, start_time=self.time, end_time=end_time))
        
        self.resume()
        
    def resume(self):
        if not self.running:
            return
        
        if self.paused:
            self.paused = False
            self.input.resume()
            for output in self.outputs:
                output.resume()
            print('Simulation resumed.')
        
        t0 = perf_counter()
        
        while self.time < self.end_time and self.running and not self.paused:
            self.step()
            
        self.duration += perf_counter() - t0
        
        if self.paused and self.running:
            self.input.pause()
            for output in self.outputs:
                output.pause()
            print('Simulation paused.')
            return
        
        aborted = not self.running
        
        self.stop()
        
        self.input.stop()
        for output in self.outputs:
            output.stop()
        
        if aborted:
            print('Simulation canceled after {duration} seconds.'.format(duration=self.duration))
        else:
            print('Simulation finished after {duration} seconds.'.format(duration=self.duration))
        
        
    def step(self):
        self.input.step()
        for output in self.outputs:
            output.step()
    
    def stop(self):
        self.running = False
        
    def pause(self):
        if self.running:
            self.paused = True
    
    def __str__(self):
     return 'time={time},world_objects={world_objects}'.format(
             time=self.time,world_objects=len(self.world_objects))

class WorldInput(object):
    
    world = None
    
    def __init__(self, world):
        self.world = world
    
    def start(self):
        pass
    
    def step(self):
        pass
    
    def stop(self):
        pass

    def resume(self):
        pass

    def pause(self):
        pass

class WorldOutput(object):
    
    world = None
    
    def __init__(self, world):
        self.world = world
    
    def start(self):
        pass
    
    def step(self):
        pass
    
    def stop(self):
        pass

    def resume(self):
        pass

    def pause(self):
        pass
        
class WorldObject(object):    
    '''Definition für alle Punktmassen in World die von Physik-Engine berechnet werden'''
    
    position = None
    velocity = None
    force = None
    mass = None
    
    def __init__(self,
                 mass = 0.0,
                 position = np.zeros(3), 
                 velocity = np.zeros(3),
                 force = np.zeros(3)):
        self.position = position
        self.velocity = velocity
        self.force = force
        self.mass = mass
    
    def __str__(self):
     return 'mass={mass},position={position},velocity={velocity},force={force}'.format(
             mass=self.mass, position=self.position, velocity=self.velocity, force=self.force)
    
class Sphere(WorldObject):    
    '''Definition für perfekt spherische Objekte in World'''
    
    radius = None
    
    def __init__(self,
                 mass = 0.0,
                 position = np.zeros(3), 
                 velocity = np.zeros(3),
                 force = np.zeros(3),
                 radius = 0.0):
        super(Sphere, self).__init__(mass, position, velocity, force)
        self.radius = radius
    
    def __str__(self):
     return super(Sphere, self).__str__() + ',radius={radius}'.format(radius=self.radius)