# -*- coding: utf-8 -*-
from time import sleep, perf_counter
from vpython import *
import numpy as np
import sys
import math
from model import WorldOutput

class RenderEngine(WorldOutput):
    
    time = 0
    last_step = 0

    time_scale = None
    radius_scale = None
    min_step_interval = 0.1
    
    def __init__(self, world, time_scale, radius_scale):
        super(RenderEngine, self).__init__(world)
        self.time_scale = time_scale
        self.radius_scale = radius_scale
    
    def step(self):
        dt = self.world.time - self.time
        if dt >= self.min_step_interval:
            sleep(dt * self.time_scale)
            self.time = self.world.time
            self.draw()
            self.last_step = perf_counter()
    
    def draw(self):
        pass

class RenderEngineVPython(RenderEngine):
    
    render_objects = {}
    ro_labels = {}
    colors = [color.yellow, vector(0.5,0.5,0.5), vector(0.818,0.716,0.481), vector(0.0,0.420,0.150), vector(0.871,0.310,0.177), vector(0.556,0.376,0.253), vector(0.761,0.706,0.709), vector(0.700,0.923,0.957), vector(0.332,0.391,1), vector(1,0.7,0.2)]
    ro_textures = ["textures/larsun_1_1.jpg", "textures/mercury.jpg", "textures/venus.jpg", "textures/earth.jpg", "textures/mars.jpg", "textures/jupiter.jpg", "textures/saturn.jpg", "textures/uranus.jpg", "textures/neptune.jpg", textures.stucco, textures.rug]
    #def __init__(self, time_scale, radius_scale, min_step_interval):
    #    super(RenderEngineVPython, self).__init__(time_scale, radius_scale, min_step_interval)
    scene = None
    halt = False
    halted = False

    def key_evt(self, ev):
        if ev.key == ' ':
            if not self.halt and not self.halted:
                self.halt = True
                self.world.pause()
            elif self.halted and self.halt:
                self.halt = False
                self.halted = False
                self.world.resume()
        elif ev.key == '.' and self.halt:
            self.world.step()


    def start(self):
        count_index = 0
        self.scene = canvas(width=1000, height=500, resizable=True)
        self.scene.bind('keydown', self.key_evt)
        for key, wo in self.world.world_objects.items():
            pos = wo.position
            r = math.log(wo.radius) * self.radius_scale
            self.render_objects[key] = sphere(pos = vector(pos[0],pos[1],pos[2]), radius = r, texture = self.ro_textures[count_index], make_trail=True, trail_type="points", trail_radius = r/5, trail_color = self.colors[count_index], retain = 1000, interval = 5)
            self.ro_labels[key] = label(pos = vector(pos[0],pos[1],pos[2]), text = key, yoffset = 20, height = 10, font = 'sans')
            count_index += 1

    def pause(self):
        self.halted = True

    def draw(self):
        name_index = 0
        years = int(self.world.time / 86400 / 365)
        days = int(self.world.time / 86400) % 365
        seconds = int(self.world.time % 86400)
        self.scene.caption = 'years: {years:03d}  days: {days:03d}  seconds: {seconds:05d}  total seconds: {total_seconds}'.format(
            years=years, days=days, seconds=seconds, total_seconds=self.world.time
        )
        for key, wo in self.world.world_objects.items():
           pos = wo.position
           radius = wo.radius * self.radius_scale
           ro = self.render_objects[key]
           ro.pos = vector(pos[0],pos[1],pos[2])
           rol = self.ro_labels[key]
           rol.pos = ro.pos
           name_index += 1

if __name__ == '__main__':
    world = FileWorld(sys.argv[1])

    engine = RenderEngineVPython(world)
    engine.start()
